@@ -2,13 +2,14 @@ import importlib
 import importlib.util
 import inspect
 from copy import deepcopy
-import os
+import dask
+from dask.distributed import Client
 import numpy as np
 from sklearn.base import BaseEstimator
 from sklearn.model_selection._search import ParameterGrid
 
 from photonai.base.photon_pipeline import PhotonPipeline
-from photonai.base.registry.element_dictionary import ElementDictionary
+from photonai.base.registry.registry import PhotonRegistry
 from photonai.helper.helper import PhotonDataHelper
 from photonai.optimization.config_grid import create_global_config_grid, create_global_config_dict
 from photonai.photonlogger.logger import logger
@@ -44,8 +45,6 @@ class PipelineElement(BaseEstimator):
         Any parameters that should be passed to the object to be instantiated, default parameters
 
     """
-    # Registering Pipeline Elements
-    ELEMENT_DICTIONARY = ElementDictionary.get_package_info()
 
     def __init__(self, name, hyperparameters: dict = None, test_disabled: bool = False,
                  disabled: bool = False, base_element=None, batch_size=0, **kwargs):
@@ -60,9 +59,18 @@ class PipelineElement(BaseEstimator):
             hyperparameters = {}
 
         if base_element is None:
-            if name in PipelineElement.ELEMENT_DICTIONARY:
+
+            # Registering Pipeline Elements
+            if len(PhotonRegistry.ELEMENT_DICTIONARY) == 0:
+                registry = PhotonRegistry
+
+            if name not in PhotonRegistry.ELEMENT_DICTIONARY:
+                # try to reload
+                PhotonRegistry.ELEMENT_DICTIONARY = PhotonRegistry().get_package_info()
+
+            if name in PhotonRegistry.ELEMENT_DICTIONARY:
                 try:
-                    desired_class_info = PipelineElement.ELEMENT_DICTIONARY[name]
+                    desired_class_info = PhotonRegistry.ELEMENT_DICTIONARY[name]
                     desired_class_home = desired_class_info[0]
                     desired_class_name = desired_class_info[1]
                     imported_module = importlib.import_module(desired_class_home)
@@ -70,9 +78,10 @@ class PipelineElement(BaseEstimator):
                     self.base_element = desired_class(**kwargs)
                 except AttributeError as ae:
                     logger.error('ValueError: Could not find according class:'
-                                   + str(PipelineElement.ELEMENT_DICTIONARY[name]))
-                    raise ValueError('Could not find according class:', PipelineElement.ELEMENT_DICTIONARY[name])
+                                   + str(PhotonRegistry.ELEMENT_DICTIONARY[name]))
+                    raise ValueError('Could not find according class:', PhotonRegistry.ELEMENT_DICTIONARY[name])
             else:
+                # if even after reload the element does not appear, it is not supported
                 logger.error('Element not supported right now:' + name)
                 raise NameError('Element not supported right now:', name)
         else:
@@ -212,7 +221,7 @@ class PipelineElement(BaseEstimator):
 
         if already_existing_element_with_that_name > 0:
             error_msg = "Already added a pipeline element with the name " + pipe_element.name + " to " + self.name
-            logger.warn(error_msg)
+            logger.warning(error_msg)
 
             # check for other items that have been renamed
             nr_of_existing_elements_with_that_name = len([i for i in self.elements if i.name.startswith(pipe_element.name)])
@@ -221,14 +230,14 @@ class PipelineElement(BaseEstimator):
                 nr_of_existing_elements_with_that_name += 1
                 new_name = pipe_element.name + str(nr_of_existing_elements_with_that_name + 1)
 
-            logger.warn("Renaming " + pipe_element.name + " in " + self.name + " to " + new_name + " in " + self.name)
+            logger.warning("Renaming " + pipe_element.name + " in " + self.name + " to " + new_name + " in " + self.name)
             pipe_element.name = new_name
 
         self.elements.append(pipe_element)
         return self
 
     def copy_me(self):
-        if self.name in self.ELEMENT_DICTIONARY:
+        if self.name in PhotonRegistry.ELEMENT_DICTIONARY:
             # we need initial name to refer to the class to be instantiated  (SVC) even though the name might be SVC2
             copy = PipelineElement(self.initial_name, {}, test_disabled=self.test_disabled,
                                    disabled=self.disabled, batch_size=self.batch_size, **self.kwargs)
@@ -341,7 +350,9 @@ class PipelineElement(BaseEstimator):
 
     def __batch_predict(self, delegate, X, **kwargs):
         if not isinstance(X, list) and not isinstance(X, np.ndarray):
-            logger.warn("Cannot do batching on a single entity.")
+            warning = "Cannot do batching on a single entity."
+            logger.warning(warning)
+            raise Warning(warning)
             return delegate(X, **kwargs)
 
             # initialize return values
@@ -365,9 +376,6 @@ class PipelineElement(BaseEstimator):
     def __predict(self, X, **kwargs):
         if not self.disabled:
             if hasattr(self.base_element, 'predict'):
-                # Todo: check if element has kwargs, and give it to them
-                # todo: so this todo above was old, here are my changes:
-                #return self.base_element.predict(X)
                 return self.adjusted_predict_call(self.base_element.predict, X, **kwargs)
             else:
                 logger.error('BaseException. base Element should have function ' +
@@ -403,7 +411,6 @@ class PipelineElement(BaseEstimator):
                 #return self.base_element.predict_proba(X)
                 return self.adjusted_predict_call(self.base_element.predict_proba, X, **kwargs)
             else:
-
                 # todo: in case _final_estimator is a Branch, we do not know beforehand it the base elements will
                 #  have a predict_proba -> if not, just return None (@Ramona, does this make sense?)
                 # logger.error('BaseException. base Element should have "predict_proba" function.')
@@ -443,7 +450,9 @@ class PipelineElement(BaseEstimator):
 
     def __batch_transform(self, X, y=None, **kwargs):
         if not isinstance(X, list) and not isinstance(X, np.ndarray):
-            logger.warn("Cannot do batching on a single entity.")
+            warning = "Cannot do batching on a single entity."
+            logger.warning(warning)
+            raise Warning(warning)
             return self.__transform(X, y, **kwargs)
 
             # initialize return values
@@ -1270,3 +1279,158 @@ class CallbackElement(PhotonNative):
     @property
     def feature_importances_(self):
         return
+
+
+class ParallelBranch(Branch):
+    """
+    A substream of elements, that do not need fit() but can instantly use transform on a single subject level,
+    and therefore can be computed in parallel.
+
+    Parameters
+    ----------
+    * `name` [str]:
+        Name of the parallelizable pipeline branch
+
+    """
+
+    def __init__(self, name, nr_of_processes=1, output_img: bool = False):
+        Branch.__init__(self, name)
+
+        self._nr_of_processes = 1
+        self.local_cluster = None
+        self.client = None
+        self.nr_of_processes = nr_of_processes
+
+        self.has_hyperparameters = True
+        self.needs_y = False
+        self.needs_covariates = True
+        self.current_config = None
+
+    def __del__(self):
+        if self.local_cluster is not None:
+            self.local_cluster.close()
+
+    def fit(self, X, y=None, **kwargs):
+        # do nothing here!!
+        return self
+
+    @property
+    def nr_of_processes(self):
+        return self._nr_of_processes
+    # Todo : !
+    # @classmethod
+    # def set_local_cluster(cls, nr_of_processes):
+    #     cls.local_cluster =
+
+    @nr_of_processes.setter
+    def nr_of_processes(self, value):
+        self._nr_of_processes = value
+        if self._nr_of_processes > 1:
+            if self.local_cluster is None:
+                self.local_cluster = Client(threads_per_worker=1,
+                                            n_workers=self.nr_of_processes,
+                                            processes=False)
+            else:
+                self.local_cluster.n_workers = self.nr_of_processes
+        else:
+            self.local_cluster = None
+
+    def __iadd__(self, pipe_element):
+        """
+        Add an element to the neuro branch. Only neuro pipeline elements are allowed.
+        Returns self
+
+        Parameters
+        ----------
+        * `pipe_element` [PipelineElement]:
+            The transformer object to add. Should be registered in the Neuro module.
+        """
+
+        self.elements.append(pipe_element)
+        self._prepare_pipeline()
+
+        return self
+
+    def transform(self, X, y=None, **kwargs):
+
+        if self.base_element.cache_folder is not None:
+            # make sure we cache individually
+            self.base_element.single_subject_caching = True
+            self.base_element.caching = True
+        if self.nr_of_processes > 1:
+
+            if self.base_element.cache_folder is not None:
+                # at first apply the transformation on several cores, everything gets written to the cache,
+                # so the next step only has to reload the data ...
+                self.apply_transform_parallelized(X)
+            else:
+                logger.error("Cannot use parallelization without a cache folder specified in the hyperpipe."
+                               "Using single core instead")
+
+            logger.debug('ParallelBranch ' + self.name + ' is collecting data from the different cores...')
+        X_new, _, _ = self.base_element.transform(X)
+
+        return X_new, y, kwargs
+
+    def set_params(self, **kwargs):
+        self.current_config = kwargs
+        super(ParallelBranch, self).set_params(**kwargs)
+
+    def copy_me(self):
+        new_copy = super().copy_me()
+        new_copy.base_element.current_config = self.base_element.current_config
+        new_copy.base_element.single_subject_caching = True
+        new_copy.base_element.cache_folder = self.base_element.cache_folder
+        new_copy.local_cluster = self.local_cluster
+        new_copy.nr_of_processes = self.nr_of_processes
+
+        # todo: clarify this with Ramona
+        new_copy.do_not_delete_cache_folder = True
+
+        return new_copy
+
+    def inverse_transform(self, X, y=None, **kwargs):
+        for transform in self.elements[::-1]:
+            if hasattr(transform, 'inverse_transform'):
+                X, y, kwargs = transform.inverse_transform(X, y, **kwargs)
+            else:
+                return X, y, kwargs
+        return X, y, kwargs
+
+    @staticmethod
+    def parallel_application(pipe_copy, data):
+        pipe_copy.transform(data)
+        return
+
+    def apply_transform_parallelized(self, X):
+        """
+
+        :param X: the data to which the delegate should be applied in parallel
+        """
+
+        if self.nr_of_processes > 1:
+
+            jobs_to_do = list()
+
+            # distribute the data equally to all available cores
+            number_of_items_to_process = PhotonDataHelper.find_n(X)
+            number_of_items_for_each_core = int(np.ceil(number_of_items_to_process / self.nr_of_processes))
+            logger.info('ParallelBranch ' + self.name +
+                         ': Using ' + str(self.nr_of_processes) + ' cores calculating ' + str(number_of_items_for_each_core)
+                         + ' items each')
+            for start, stop in PhotonDataHelper.chunker(number_of_items_to_process, number_of_items_for_each_core):
+                X_batched, _, _ = PhotonDataHelper.split_data(X, None, {}, start, stop)
+
+                # copy my pipeline
+                new_pipe_mr = self.copy_me()
+                new_pipe_copy = new_pipe_mr.base_element
+                new_pipe_copy.cache_folder = self.base_element.cache_folder
+                new_pipe_copy.skip_loading = True
+                new_pipe_copy._parallel_use = True
+
+                del_job = dask.delayed(ParallelBranch.parallel_application)(new_pipe_copy, X_batched)
+                jobs_to_do.append(del_job)
+
+            dask.compute(*jobs_to_do)
+
+
