@@ -1,17 +1,17 @@
 import inspect
+import importlib
 import json
 import os
 import sys
-from os import path
+from glob import glob
 
 import numpy as np
 from sklearn.datasets import load_breast_cancer, load_boston
+from shutil import copyfile
 
-from photonai.base.photon_elements import PipelineElement
 from photonai.photonlogger.logger import logger
 
 
-
 class PhotonRegistry:
     """
     Helper class to manage the PHOTON Element Register.
@@ -44,52 +44,164 @@ class PhotonRegistry:
 
     """
 
-    PHOTON_REGISTRIES = ['PhotonCore', 'PhotonNeuro']
+    ELEMENT_DICTIONARY = dict()
+    PHOTON_REGISTRIES = ['PhotonCore']
+    CUSTOM_ELEMENTS_FOLDER = None
+    CUSTOM_ELEMENTS = None
 
     def __init__(self, custom_elements_folder: str = None):
-        if custom_elements_folder:
-            self._load_custom_folder(custom_elements_folder)
+
+        self.current_folder = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
+        self.module_path = os.path.join(self.current_folder, "modules")
+        if not os.path.isdir(self.module_path):
+            os.mkdir(self.module_path)
+
+        # update list with available sub_elements
+        self._list_available_modules()
+        PhotonRegistry.CUSTOM_ELEMENTS_FOLDER = custom_elements_folder
+        self._load_custom_folder(custom_elements_folder)
+
+        if len(PhotonRegistry.ELEMENT_DICTIONARY) == 0:
+            PhotonRegistry.ELEMENT_DICTIONARY = self.get_package_info()
+
+    def _list_available_modules(self):
+        for abs_filename in glob(os.path.join(self.module_path, "*.json")):
+            basename = os.path.basename(abs_filename)
+            file, ext = os.path.splitext(basename)
+            if file not in PhotonRegistry.PHOTON_REGISTRIES:
+                PhotonRegistry.PHOTON_REGISTRIES.append(file)
+
+    def add_module(self, path_to_file: str):
+        filename = os.path.basename(path_to_file)
+        copyfile(path_to_file, os.path.join(self.module_path, filename))
+        self._list_available_modules()
+        PhotonRegistry.ELEMENT_DICTIONARY = self.get_package_info()
+
+    def delete_module(self, module_name: str):
+        PhotonRegistry.PHOTON_REGISTRIES.remove(module_name)
+        os.remove(os.path.join(self.module_path, module_name + ".json"))
+        PhotonRegistry.ELEMENT_DICTIONARY = self.get_package_info()
+
+    def _load_json(self, photon_package: str):
+        """
+        Load JSON file in which the elements for the PHOTON submodule are stored.
+
+        The JSON files are stored in the framework folder by the name convention 'photon_package.json'
+
+        Parameters:
+        -----------
+        * 'photon_package' [str]:
+          The name of the photonai submodule
+
+        Returns:
+        --------
+        JSON file as dict, file path as str
+        """
+
+        if photon_package == 'CustomElements':
+            folder = PhotonRegistry.CUSTOM_ELEMENTS_FOLDER
+            if not folder:
+                return {}
+        elif photon_package == "PhotonCore":
+            folder = self.current_folder
         else:
-            self.custom_elements = None
-            self.custom_elements_folder = None
-            self.custom_elements_file = None
+            folder = self.module_path
+
+        file_name = os.path.join(folder, photon_package + '.json')
+        file_content = {}
+
+        # Reading json
+        try:
+            with open(file_name, 'r') as f:
+                try:
+                    file_content = json.load(f)
+                except json.JSONDecodeError as jde:
+                    # handle empty file
+                    if jde.msg == 'Expecting value':
+                        logger.error("Package File " + file_name + " was empty.")
+                    else:
+                        raise jde
+        except FileNotFoundError:
+            logger.error("Could not find file for " + photon_package)
+        if not file_content:
+            file_content = dict()
+        return file_content
+
+    def get_package_info(self, photon_package: list = PHOTON_REGISTRIES) -> dict:
+        """
+        Collect all registered elements from JSON file
+
+        Parameters:
+        -----------
+        * 'photon_package' [list]:
+          The names of the PHOTONAI submodules for which the elements should be retrieved
+
+        Returns
+        -------
+        Dict of registered elements
+        """
+        class_info = dict()
+        for package in photon_package:
+
+            content = self._load_json(package)
+
+            for idx, key in enumerate(content):
+                class_path, class_name = os.path.splitext(content[key][0])
+
+                if idx == 0 and package not in ["PhotonCore", "CustomElements"]:
+                    # try to import something from module.
+                    # if that fails. drop this shit.
+                    try:
+                        imported_module = importlib.import_module(class_path)
+                        desired_class = getattr(imported_module, class_name[1:])
+                        custom_element = desired_class()
+                    except (AttributeError, ModuleNotFoundError) as e:
+                        logger.error(e)
+                        logger.error("Could not import from package {}. Deleting json.".format(package))
+                        self.delete_module(package)
+
+                class_info[key] = class_path, class_name[1:]
+        return class_info
 
     def _load_custom_folder(self, custom_elements_folder):
-        self.custom_elements_folder, self.custom_elements_file = self._check_custom_folder(custom_elements_folder)
 
-        # load custom elements from json
-        self.custom_elements = self._load_json('CustomElements')
-        self.PHOTON_REGISTRIES.append('CustomElements')
+        if PhotonRegistry.CUSTOM_ELEMENTS_FOLDER is not None:
+            PhotonRegistry.CUSTOM_ELEMENTS_FOLDER = self._check_custom_folder(custom_elements_folder)
+
+            # load custom elements from json
+            PhotonRegistry.CUSTOM_ELEMENTS = self._load_json('CustomElements')
+            PhotonRegistry.PHOTON_REGISTRIES.append('CustomElements')
+            PhotonRegistry.ELEMENT_DICTIONARY.update(PhotonRegistry.CUSTOM_ELEMENTS)
 
     @staticmethod
     def _check_custom_folder(custom_folder):
-        if not path.exists(custom_folder):
+        if not os.path.exists(custom_folder):
             logger.info('Creating folder {}'.format(custom_folder))
             os.makedirs(custom_folder)
 
-        custom_file = path.join(custom_folder, 'CustomElements.json')
-        if not path.isfile(custom_file):
+        custom_file = os.path.join(custom_folder, 'CustomElements.json')
+        if not os.path.isfile(custom_file):
             logger.info('Creating CustomElements.json')
             with open(custom_file, 'w') as f:
                 json.dump('', f)
 
-        return custom_folder, custom_file
+        return custom_folder
 
     def activate(self):
-        if not self.custom_elements_folder:
+        if not PhotonRegistry.CUSTOM_ELEMENTS_FOLDER:
             raise ValueError("To activate a custom elements folder, specify a folder when instantiating the registry "
                              "module. Example: registry = PhotonRegistry('/MY/CUSTOM/ELEMENTS/FOLDER) "
                              "In case you don't have any custom models, there is no need to activate the registry.")
-        if not path.exists(self.custom_elements_folder):
-            raise FileNotFoundError("Couldn't find custom elements folder: {}".format(self.custom_elements_folder))
-        if not path.isfile(path.join(self.custom_elements_folder, 'CustomElements.json')):
+        if not os.path.exists(PhotonRegistry.CUSTOM_ELEMENTS_FOLDER):
+            raise FileNotFoundError("Couldn't find custom elements folder: {}".format(PhotonRegistry.CUSTOM_ELEMENTS_FOLDER))
+        if not os.path.isfile(os.path.join(PhotonRegistry.CUSTOM_ELEMENTS_FOLDER, 'CustomElements.json')):
             raise FileNotFoundError("Couldn't find CustomElements.json. Did you register your element first?")
 
         # add folder to python path
         logger.info("Adding custom elements folder to system path...")
-        sys.path.append(self.custom_elements_folder)
+        sys.path.append(PhotonRegistry.CUSTOM_ELEMENTS_FOLDER)
 
-        PipelineElement.ELEMENT_DICTIONARY.update(self._get_package_info(['CustomElements']))
+        PhotonRegistry.ELEMENT_DICTIONARY.update(self.get_package_info(['CustomElements']))
         logger.info('Successfully activated custom elements!')
 
     def register(self, photon_name: str, class_str: str, element_type: str):
@@ -109,25 +221,25 @@ class PhotonRegistry:
         """
 
         # check if folder exists
-        if not self.custom_elements_folder:
+        if not PhotonRegistry.CUSTOM_ELEMENTS_FOLDER:
             raise ValueError("To register an element, specify a custom elements folder when instantiating the registry "
                              "module. Example: registry = PhotonRegistry('/MY/CUSTOM/ELEMENTS/FOLDER)")
 
         if not element_type == "Estimator" and not element_type == "Transformer":
             raise ValueError("Variable element_type must be 'Estimator' or 'Transformer'")
 
-        duplicate = self._check_duplicate(photon_name=photon_name, class_str=class_str, content=self.custom_elements)
+        duplicate = self._check_duplicate(photon_name=photon_name, class_str=class_str, content=PhotonRegistry.CUSTOM_ELEMENTS)
 
         if not duplicate:
-            python_file = path.join(self.custom_elements_folder, class_str.split('.')[0] + '.py')
-            if not path.isfile(python_file):
+            python_file = os.path.join(PhotonRegistry.CUSTOM_ELEMENTS_FOLDER, class_str.split('.')[0] + '.py')
+            if not os.path.isfile(python_file):
                 raise FileNotFoundError("Couldn't find python file {} in your custom elements folder. "
                                         "Please copy your file into this folder first!".format(python_file))
             # add new element
-            self.custom_elements[photon_name] = class_str, element_type
+            PhotonRegistry.CUSTOM_ELEMENTS[photon_name] = class_str, element_type
 
             # write back to file
-            self._write2json(self.custom_elements)
+            self._write_to_json(PhotonRegistry.CUSTOM_ELEMENTS)
             logger.info('Adding PipelineElement ' + class_str + ' to CustomElements.json as "' + photon_name + '".')
 
             # activate custom elements
@@ -140,8 +252,11 @@ class PhotonRegistry:
             logger.error('Could not register element!')
 
     def _run_tests(self, photon_name, element_type):
-        # check import
-        custom_element = PipelineElement(photon_name)
+
+        # this is a sneaky hack to avoid circular imports of PipelineElement
+        imported_module = importlib.import_module("photonai.base")
+        desired_class = getattr(imported_module, "PipelineElement")
+        custom_element = desired_class(photon_name)
 
         # check if has fit, transform, predict
         if not hasattr(custom_element.base_element, 'fit'):
@@ -155,12 +270,12 @@ class PhotonRegistry:
 
         # check if estimator is regressor or classifier
         if element_type == 'Estimator':
-            if hasattr(custom_element.base_element, '_estimator_type'):
-                est_type = getattr(custom_element.base_element, '_estimator_type')
+            if hasattr(custom_element, '_estimator_type'):
+                est_type = getattr(custom_element, '_estimator_type')
                 if est_type == "regressor":
-                    X, y = load_boston(True)
+                    X, y = load_boston(return_X_y=True)
                 elif est_type == "classifier":
-                    X, y = load_breast_cancer(True)
+                    X, y = load_breast_cancer(return_X_y=True)
                 else:
                     raise ValueError("Custom element does not specify whether it is a regressor or classifier. "
                                      "Is {}".format(est_type))
@@ -169,7 +284,7 @@ class PhotonRegistry:
                                           "Consider inheritance from ClassifierMixin or RegressorMixin or set "
                                           "_estimator_type manually.")
         else:
-            X, y = load_boston(True)
+            X, y = load_boston(return_X_y=True)
 
         # try and test functionality
         kwargs = {'covariates': np.random.randn(len(y))}
@@ -242,7 +357,7 @@ class PhotonRegistry:
         * 'photon_name' [str]:
           The string literal which accesses the class
         """
-        content = self._get_package_info()  # load existing json
+        content = self.get_package_info()  # load existing json
 
         if photon_name in content:
             element_namespace, element_name = content[photon_name]
@@ -277,10 +392,10 @@ class PhotonRegistry:
           The string literal encoding the class
         """
 
-        if photon_name in self.custom_elements:
-            del self.custom_elements[photon_name]
+        if photon_name in PhotonRegistry.CUSTOM_ELEMENTS:
+            del PhotonRegistry.CUSTOM_ELEMENTS[photon_name]
 
-            self._write2json(self.custom_elements)
+            self._write_to_json(PhotonRegistry.CUSTOM_ELEMENTS)
             logger.info('Removing the PipelineElement named "{0}" from CustomElements.json.'.format(photon_name))
         else:
             logger.info('Cannot remove "{0}" from CustomElements.json. Element has not been registered before.'.format(photon_name))
@@ -315,47 +430,7 @@ class PhotonRegistry:
             return True
         return False
 
-    def _load_json(self, photon_package: str):
-        """
-        Load JSON file in which the elements for the PHOTON submodule are stored.
-
-        The JSON files are stored in the framework folder by the name convention 'photon_package.json'
-
-        Parameters:
-        -----------
-        * 'photon_package' [str]:
-          The name of the photonai submodule
-
-        Returns:
-        --------
-        JSON file as dict, file path as str
-        """
-
-        if photon_package == 'CustomElements':
-            folder = self.custom_elements_folder
-            if not folder:
-                return {}
-        else:
-            folder = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
-
-        file_name = path.join(folder, photon_package + '.json')
-        file_content = {}
-
-        # Reading json
-        with open(file_name, 'r') as f:
-            try:
-                file_content = json.load(f)
-            except json.JSONDecodeError as jde:
-                # handle empty file
-                if jde.msg == 'Expecting value':
-                    logger.error("Package File " + file_name + " was empty.")
-                else:
-                    raise jde
-        if not file_content:
-            file_content = dict()
-        return file_content
-
-    def _write2json(self, content2write: dict):
+    def _write_to_json(self, content_to_write: dict):
         """
         Write json content to file
 
@@ -367,31 +442,8 @@ class PhotonRegistry:
           The PHOTON submodule name to which the new class belongs, so it is written to the correct json file
         """
         # Writing JSON data
-        with open(self.custom_elements_file, 'w') as f:
-            json.dump(content2write, f)
-
-    def _get_package_info(self, photon_package: list = PHOTON_REGISTRIES) -> dict:
-        """
-        Collect all registered elements from JSON file
-
-        Parameters:
-        -----------
-        * 'photon_package' [list]:
-          The names of the PHOTON submodules for which the elements should be retrieved
-
-        Returns
-        -------
-        Dict of registered elements
-        """
-        class_info = dict()
-        for package in photon_package:
-
-            content = self._load_json(package)
-
-            for key in content:
-                class_path, class_name = os.path.splitext(content[key][0])
-                class_info[key] = class_path, class_name[1:]
-        return class_info
+        with open(os.path.join(self.CUSTOM_ELEMENTS_FOLDER, "CustomElements.json"), 'w') as f:
+            json.dump(content_to_write, f)
 
     def list_available_elements(self, photon_package=PHOTON_REGISTRIES):
         """
