@@ -3,18 +3,14 @@ from sklearn.base import BaseEstimator, TransformerMixin
 
 class SourceSplitter(BaseEstimator, TransformerMixin):
     """
-    Source splitter
+    SourceSplitter transforms data by reducing to data[:, column_indices].
     """
 
     def __init__(self, column_indices: list):
         self.column_indices = column_indices
 
     def fit(self, X, y):
-        return
+        return self
 
     def transform(self, X):
-        return X[:,self.column_indices]
-
-    def fit_transform(self, X):
-        return self.transform(X)
-
+        return X[:, self.column_indices]
