@@ -5,10 +5,12 @@ import warnings
 import datetime
 import numpy as np
 
+from typing import Union, List
+
 from photonai.helper.helper import PhotonPrintHelper, PhotonDataHelper, print_double_metrics
+from photonai.optimization.performance_constraints import PhotonBaseConstraint
 from photonai.photonlogger.logger import logger
 from photonai.processing.metrics import Scorer
-from photonai.optimization import FloatRange
 from photonai.processing.results_structure import MDBHelper, MDBInnerFold, MDBScoreInformation, MDBFoldMetric, \
     FoldOperations, MDBConfig
 
@@ -24,7 +26,7 @@ class InnerFoldManager(object):
 
     def __init__(self, pipe_ctor, specific_config: dict, optimization_infos,
                  cross_validation_infos, outer_fold_id,
-                 optimization_constraints: list = None,
+                 optimization_constraints: Union[PhotonBaseConstraint, List[PhotonBaseConstraint]] = None,
                  raise_error: bool = False,
                  training: bool = False,
                  cache_folder=None,
@@ -132,16 +134,16 @@ class InnerFoldManager(object):
                     break_cv = 0
                     for cf in self.optimization_constraints:
                         if not cf.shall_continue(config_item):
-                            logger.info(
-                                'Skipped further cross validation after fold ' + str(fold_nr) + ' due to performance constraints in ' + cf.metric)
+                            logger.info('Skipped further cross validation after fold ' + str(fold_nr) +
+                                        ' due to performance constraints in ' + cf.metric)
                             break_cv += 1
                             break
                     if break_cv > 0:
                         break
-                elif self.optimization_constraints is not None:
+                elif isinstance(self.optimization_constraints, PhotonBaseConstraint):
                     if not self.optimization_constraints.shall_continue(config_item):
-                        logger.info(
-                            'Skipped further cross validation after fold ' + str(fold_nr) + ' due to performance constraints in ' + cf.metric)
+                        logger.info('Skipped further cross validation after fold ' + str(fold_nr) +
+                                    ' due to performance constraints in ' + self.optimization_constraints.metric)
                         break
 
             InnerFoldManager.process_fit_results(config_item,
@@ -333,17 +335,6 @@ class InnerFoldManager(object):
         scoring_time_start = time.time()
 
         output_metrics = {}
-        non_default_score_metrics = list(metrics)
-        # that does not work because it is not an exact match and also reacts to e.g. f1_score
-        # if 'score' in metrics:
-        # so we use this:
-        checklist = ['score']
-        matches = set(checklist).intersection(set(non_default_score_metrics))
-        if len(matches) > 0:
-            # Todo: Here it is potentially slowing down!!!!!!!!!!!!!!!!
-            default_score = estimator.score(X, y_true)
-            output_metrics['score'] = default_score
-            non_default_score_metrics.remove('score')
 
         if not training:
             y_pred = estimator.predict(X, **kwargs)
@@ -365,10 +356,10 @@ class InnerFoldManager(object):
                     msg = "If scorer object does not return 1d array or list, PHOTON expected name 'y_pred' in nd array."
                     logger.error(msg)
                     raise KeyError(msg)
-                score_metrics = Scorer.calculate_metrics(y_true, y_pred["y_pred"], non_default_score_metrics)
+                score_metrics = Scorer.calculate_metrics(y_true, y_pred["y_pred"], metrics)
             else:
                 y_pred_names = []
-                score_metrics = Scorer.calculate_metrics(y_true, y_pred, non_default_score_metrics)
+                score_metrics = Scorer.calculate_metrics(y_true, y_pred, metrics)
 
             # add default metric
             if output_metrics:
