@@ -79,23 +79,22 @@ class PermutationTest:
                 self.pipe.permutation_id = self.mother_permutation_id
                 self.pipe.fit(X, y_true, **kwargs)
                 self.pipe.results.computation_completed = True
-
                 self.pipe.results.permutation_test = MDBPermutationResults(n_perms=self.n_perms)
-                self.pipe.results.save()
+                self.clear_data_and_save(self.pipe)
                 existing_reference = self.pipe.results
 
             except Exception as e:
                 if self.pipe.results is not None:
                     self.pipe.results.permutation_failed = str(e)
-                    self.pipe.results.save()
+                    logger.error(e)
+                    PermutationTest.clear_data_and_save(self.pipe)
+                raise e
 
         # check for sanity
         if not self.__validate_usability(existing_reference):
             raise RuntimeError("Permutation Test is not adviced because results are not better than dummy. Aborting.")
 
         # find how many permutations have been computed already:
-        # existing_permutations = MDBHyperpipe.objects.raw({'permutation_id': self.permutation_id,
-        #                                                   'computation_completed': True}).count()
         existing_permutations = list(MDBHyperpipe.objects.raw({'permutation_id': self.permutation_id,
                                                                'computation_completed': True}).only('name'))
         existing_permutations = [int(perm_run.name.split('_')[-1]) for perm_run in existing_permutations]
@@ -140,12 +139,19 @@ class PermutationTest:
                                                                  self.permutations[perm_run],
                                                                  self.permutation_id, self.verbosity, **kwargs)
 
-        perm_result = self._calculate_results(self.permutation_id)
+        perm_result = self._calculate_results(self.permutation_id,
+                                              mongodb_path=self.pipe.output_settings.mongodb_connect_url)
 
         performance_df = pd.DataFrame(dict([(name, [i]) for name, i in perm_result.p_values.items()]))
         performance_df.to_csv(os.path.join(existing_reference.output_folder, 'permutation_test_results.csv'))
         return self
 
+    @staticmethod
+    def clear_data_and_save(perm_pipe):
+        perm_pipe.results.outer_folds = list()
+        perm_pipe.results.best_config = None
+        perm_pipe.results.save()
+
     @staticmethod
     def run_parallelized_permutation(hyperpipe_constructor, X, perm_run, y_perm, permutation_id, verbosity=-1,
                                      **kwargs):
@@ -166,9 +172,7 @@ class PermutationTest:
             print('Fitting permutation ' + str(perm_run) + ' ...')
             perm_pipe.fit(X, y_perm, **kwargs)
             perm_pipe.results.computation_completed = True
-            perm_pipe.results.outer_folds = list()
-            perm_pipe.results.best_config = None
-            perm_pipe.results.save()
+            PermutationTest.clear_data_and_save(perm_pipe)
             print('Finished permutation ' + str(perm_run) + ' ...')
         except Exception as e:
             if perm_pipe.results is not None:
@@ -178,7 +182,7 @@ class PermutationTest:
         return perm_run
 
     @staticmethod
-    def _calculate_results(permutation_id, save_to_db=True, mongodb_path="mongodb://trap-umbriel:27017/photon_results"):
+    def _calculate_results(permutation_id, save_to_db=True, mongodb_path="mongodb://localhost:27017/photon_results"):
 
         logger.info("Calculating permutation test results")
         try:
@@ -301,7 +305,7 @@ class PermutationTest:
         return mother_permutation
 
     @staticmethod
-    def prepare_for_wizard(permutation_id, wizard_id, mongo_db_connect_url="mongodb://trap-umbriel:27017/photon_results"):
+    def prepare_for_wizard(permutation_id, wizard_id, mongo_db_connect_url="mongodb://localhost:27017/photon_results"):
         mother_permutation = PermutationTest.find_reference(mongo_db_connect_url, permutation_id=wizard_id,
                                                             find_wizard_id=True)
         mother_permutation.permutation_id = PermutationTest.get_mother_permutation_id(permutation_id)
@@ -320,13 +324,17 @@ class PermutationTest:
                                            if i.metric_name == best_config_metric and i.operation == str(FoldOperations.MEAN)]
                 if len(dummy_threshold_to_beat) > 0:
                     dummy_threshold_to_beat = dummy_threshold_to_beat[0]
+                    mother_perm_threshold = [i.value for i in mother_permutation.metrics_test
+                                             if i.metric_name == best_config_metric and i.operation == str(FoldOperations.MEAN)]
+                    mother_perm_threshold = mother_perm_threshold[0]
                     if mother_permutation.hyperpipe_info.maximize_best_config_metric:
-                        if mother_permutation.best_config.best_config_score.validation.metrics[best_config_metric] > dummy_threshold_to_beat:
+
+                        if mother_perm_threshold > dummy_threshold_to_beat:
                             return True
                         else:
                             return False
                     else:
-                        if mother_permutation.best_config.best_config_score.validation.metrics[best_config_metric] < dummy_threshold_to_beat:
+                        if mother_perm_threshold < dummy_threshold_to_beat:
                             return True
                         else:
                             return False
